from __future__ import absolute_import

import cirdan
import copy
import jinja2
from cirdan.registry import registry
from contextlib import closing

class DocsResource:
    def __init__(self, api):
        self.api = api
        self.body = None

    def build(self):
        with closing(open(cirdan.template_path, 'r')) as f:
            t = jinja2.Template(f.read())

            payload = {
                "name": "Docs",
                "intro_text": "",
                "resources": copy.deepcopy(registry.api_to_resources[self.api])
            }

            if self.api in registry.api_meta:
                for key, value in registry.api_meta[self.api].items():
                    payload[key] = value

            payload["resources"] = [resource for resource in payload["resources"] if not resource.secret]
            for resource in payload["resources"]:
                resource.methods = [method for method in resource.methods if not method.secret]

            self.body = t.render(payload)

    def on_get(self, req, resp):
        if self.body is None or not cirdan.cache_template:
            self.build()
        resp.content_type = "text/html"
        resp.body = self.body
        



from __future__ import absolute_import

"""
The actual record-keeping part of Cirdan.
"""

import inspect
import itertools
import re

from collections import OrderedDict

class Resource:
    def __init__(self, cls):
        self.title = cls.__name__
        self.path = "???"
        self.description = None
        self.methods = []
        self.secret = False

    def safe_title(self):
        assert(hasattr(self, "title"))
        return re.sub("[^A-Za-z0-9_\-]", "", self.title.strip().replace(" ", "_"));

    def __str__(self):
        return "%s - %s" % (self.title, self.path)

class RouteMethod:
    def __init__(self, func):
        self.verb = METHODS_TO_VERBS[func.__name__]
        self.title = self.verb
        self.description = None
        self.parameters = []
        self.return_statuses = []
        self.content_type = None
        self.requires_permission = None
        self.secret = False
        self.example_request = None
        self.example_response = None

    def __str__(self):
        return "%s: %s" % (self.verb, str(self.title))

class Parameter:
    def __init__(self, name, description, required):
        self.name = name
        self.description = description
        self.required = required

    def __str__(self):
        return "%s: %s (required = %s" % (self.name, self.description, repr(self.required))

class ReturnStatus:
    def __init__(self, status_code, description):
        self.status_code = status_code
        self.description = description

# this is an ordered dict because I am a lazy, lazy man
METHODS_TO_VERBS = OrderedDict([
    ("on_post", "POST"),
    ("on_get", "GET"),
    ("on_put", "PUT"),
    ("on_delete", "DELETE"),
    ("on_patch", "PATCH")
])

class Registry:
    def __init__(self):
        self.api_to_resources = {}
        self.api_meta = {}
        self.resources = {}
        self.route_methods = {}

    def get(self, item):
        if inspect.isclass(item):
            if id(item) not in self.resources:
                self.resources[id(item)] = Resource(item)
            return self.resources[id(item)]
        else:
            if id(item) not in self.route_methods:
                self.route_methods[id(item)] = RouteMethod(item)
            return self.route_methods[id(item)]

    def knows_about(self, api):
        return api in self.api_to_resources

    def bind_api(self, api, resource):
        if api not in self.api_to_resources:
            self.api_to_resources[api] = []
        self.api_to_resources[api].append(resource)

    def set_api_meta(self, api, **kwargs):
        self.api_meta[api] = kwargs

    def dump(self, api):
        for resource in self.api_to_resources[api]:
            print(resource)
            for method in resource.methods:
                print("\t" + str(method))

registry = Registry()



#!/usr/bin/env python

from setuptools import setup, find_packages

setup(name="cirdan",
    version="1.1.1",
    description="Decorator-based documentation generation for Falcon-based web applications.",
    url="https://github.com/forana/python-cirdan",
    author="Alex Foran",
    author_email="alex@alexforan.com",
    packages=find_packages(),
    install_requires=[
        "Jinja2==2.7.3",
        "falcon==0.3.0"
    ],
    include_package_data=True,
    package_data={
        'cirdan': ['default.jinja2'],
        'canned_environments': ['*.json']
    }
)



from __future__ import absolute_import

"""
Resource and route decorators.
"""

import json

from cirdan.registry import registry, Parameter, ReturnStatus

# Marks the title of a resource class or route method.
def title(value):
    def inner(wrapped):
        item = registry.get(wrapped)
        item.title = value
        return wrapped
    return inner

# Marks the description of a resource class or route method.
def description(value):
    def inner(wrapped):
        item = registry.get(wrapped)
        item.description = value
        return wrapped
    return inner

# Marks that a resource class or route method should not appear in docs
def secret(wrapped):
    item = registry.get(wrapped)
    item.secret = True
    return wrapped

# Marks a parameter to a route.
def param(name, description, required = False):
    def inner(wrapped):
        route = registry.get(wrapped)
        route.parameters.append(Parameter(name, description, required))
        return wrapped
    return inner

# Marks that a route can return a certain status under a certain condition
def returns_status(status_code, description):
    def inner(wrapped):
        route = registry.get(wrapped)
        route.return_statuses.append(ReturnStatus(status_code, description))
        return wrapped
    return inner

# Marks that a route will return a certain content type under successful conditions.
def content_type(value):
    def inner(wrapped):
        route = registry.get(wrapped)
        route.content_type = value
        return wrapped
    return inner

# Marks that a route requires a certain permission
def requires_permission(value):
    def inner(wrapped):
        route = registry.get(wrapped)
        route.requires_permission = value
        return wrapped
    return inner

# Provides an example request and/or response for a route
def example(request = None, response = None):
    def inner(wrapped):
        route = registry.get(wrapped)
        if request is not None:
            route.example_request = request if isinstance(request, str) else json.dumps(request, indent = 4)
        if response is not None:
            route.example_response = response if isinstance(response, str) else json.dumps(response, indent = 4)
        return wrapped
    return inner



from __future__ import absolute_import

import falcon
import os.path

from cirdan.resource import DocsResource
from cirdan.registry import registry, METHODS_TO_VERBS

route_path = "/docs"
template_path = os.path.join(os.path.dirname(__file__), "default.jinja2")
cache_template = True

def inject():
    falcon.API._wrapped_add_route = falcon.API.add_route
    falcon.API.add_route = register

def set_meta(api, name = "Docs", intro_text = "", **kwargs):
    """
    A quick aside here - this is a function in the cirdan module instead of a
    monkey-patch into falcon because falcon uses __slots__, and thus can't have
    new properties bound.
    """
    registry.set_api_meta(api, name = name, intro_text = intro_text, **kwargs)

def register(api, path, resource):
    api._wrapped_add_route(path, resource)
    add_docs_route(api)
    cls = resource.__class__
    wrapped_resource = registry.get(cls)
    wrapped_resource.path = path
    registry.bind_api(api, wrapped_resource)
    for method, verb in METHODS_TO_VERBS.items():
        if hasattr(cls, method):
            f = getattr(cls, method)
            if hasattr(f, "__func__"): # python 2.X
                f = f.__func__
            wrapped_resource.methods.append(registry.get(f))

def add_docs_route(api):
    if not registry.knows_about(api):
        api._wrapped_add_route(route_path, DocsResource(api))
